@@ -7,7 +7,6 @@
 // std library
 use std::path::PathBuf;
 use std::time::Duration;
-use std::fs;
 
 // dependencies
 use exitfailure::ExitFailure;
@@ -19,8 +18,8 @@ use crate::error::handle_error::{
     throw_time_limit_exceeded_msg
 };
 use crate::file_handler::file::{
-    create_folder_or_error, file_exists_or_error,
-    remove_files, remove_files_with_prefix, write_file
+    create_folder_or_error, file_exists_or_error, remove_files,
+    remove_files_with_prefix, remove_folder, save_test_case
 };
 use crate::runner::types::{
     Language, is_time_limit_exceeded,
@@ -34,15 +33,12 @@ use crate::painter::style::{
     show_accepted, show_time_limit_exceeded,
     show_time_limit_exceeded_generator, show_runtime_error
 };
+
 // Constants
-use crate::constants::{
-    CACHE_FOLDER, GEN_BINARY_FILE, PREFIX_TLE_FILES,
-    QTEST_ERROR_FILE, QTEST_INPUT_FILE, QTEST_OUTPUT_FILE,
-    TARGET_BINARY_FILE, TEST_CASES_FOLDER
-};
+use crate::constants::{CACHE_FOLDER, GEN_BINARY_FILE, PREFIX_AC_FILES, PREFIX_RTE_FILES, PREFIX_TLE_FILES, QTEST_ERROR_FILE, QTEST_INPUT_FILE, QTEST_OUTPUT_FILE, TARGET_BINARY_FILE, TEST_CASES_FOLDER};
 
 pub fn run(target_file: PathBuf, gen_file: PathBuf,
-        test_cases: u32, timeout: u32, tle_break: bool, save_cases: bool) -> Result<(), ExitFailure> {
+        test_cases: u32, timeout: u32, tle_break: bool, save_bad: bool, save_all: bool) -> Result<(), ExitFailure> {
     
     // create cache folder
     create_folder_or_error(CACHE_FOLDER)?;
@@ -87,13 +83,18 @@ pub fn run(target_file: PathBuf, gen_file: PathBuf,
         return throw_compiler_error_msg("target", "<target-file>");
     }
 
-    if save_cases {
+    if save_all {
+        // Remove all previous test cases
+        remove_folder(TEST_CASES_FOLDER);
+    } else if save_bad {
         // remove test cases prefixed with test_cases/testcase_tle*.txt
         let prefix = &format!("{}/{}*", TEST_CASES_FOLDER, PREFIX_TLE_FILES)[..];
         remove_files_with_prefix(prefix);
     }
 
     let mut tle_count: u32 = 0;
+    let mut rte_count: u32 = 0;
+    let mut ac_count: u32 = 0;
 
     for test_number in 1..=test_cases {
         let response_gen = generator_file_lang.execute(timeout as u32);
@@ -116,7 +117,23 @@ pub fn run(target_file: PathBuf, gen_file: PathBuf,
         let mills_target = time_target.as_millis();
 
         if is_runtime_error(&response_target.status) {
+            rte_count += 1;
             show_runtime_error(test_number, mills_target as u32);
+            // Save the input of the test case that gave status tle
+            if save_bad || save_all {
+                // Example: test_cases/testcase_rte_1.txt
+                let file_name: &str = &format!( "{}/{}_{}.txt", TEST_CASES_FOLDER, PREFIX_RTE_FILES, rte_count)[..];
+                // save testcase
+                save_test_case(file_name, QTEST_INPUT_FILE);
+            }
+            // check if the tle_breck flag is high
+            if tle_break {
+                // remove input, output and error files
+                remove_files(vec![QTEST_INPUT_FILE, QTEST_OUTPUT_FILE, QTEST_ERROR_FILE,
+                    TARGET_BINARY_FILE, GEN_BINARY_FILE]);
+                
+               return Ok(());
+            }
             continue;
         } else if is_compiled_error(&response_target.status) {
             return throw_compiler_error_msg("target", "<target-file>");
@@ -128,39 +145,35 @@ pub fn run(target_file: PathBuf, gen_file: PathBuf,
             show_time_limit_exceeded(test_number, timeout);
 
             // Save the input of the test case that gave status tle
-            if save_cases {
-                // create test_cases folder
-                create_folder_or_error(TEST_CASES_FOLDER)?;
+            if save_bad || save_all {
                 // Example: test_cases/testcase_tle_1.txt
                 let file_name: &str = &format!( "{}/{}_{}.txt", TEST_CASES_FOLDER, PREFIX_TLE_FILES, tle_count)[..];
-                write_file(file_name, fs::read_to_string(QTEST_INPUT_FILE).unwrap().as_bytes())?;
+                // save testcase
+                save_test_case(file_name, QTEST_INPUT_FILE);
             }
             
             // check if the tle_breck flag is high
             if tle_break {
                 // remove input, output and error files
-                remove_files(vec![
-                    QTEST_INPUT_FILE,
-                    QTEST_OUTPUT_FILE,
-                    QTEST_ERROR_FILE,
-                    TARGET_BINARY_FILE,
-                    GEN_BINARY_FILE
-                ]);
+                remove_files(vec![QTEST_INPUT_FILE, QTEST_OUTPUT_FILE, QTEST_ERROR_FILE,
+                    TARGET_BINARY_FILE, GEN_BINARY_FILE]);
+                
                return Ok(());
             }
         } else {
+            ac_count += 1;
+            if save_all {
+                // Example: test_cases/testcase_ac_1.txt
+                let file_name: &str = &format!( "{}/{}_{}.txt", TEST_CASES_FOLDER, PREFIX_AC_FILES, ac_count)[..];
+                save_test_case(file_name, QTEST_INPUT_FILE);
+            }
             show_accepted(test_number, mills_target as u32);
         }
     }
 
     // remove input, output and error files
-    remove_files(vec![
-        QTEST_INPUT_FILE,
-        QTEST_OUTPUT_FILE,
-        QTEST_ERROR_FILE,
-        TARGET_BINARY_FILE,
-        GEN_BINARY_FILE
-    ]);
+    remove_files(vec![QTEST_INPUT_FILE, QTEST_OUTPUT_FILE, QTEST_ERROR_FILE,
+        TARGET_BINARY_FILE, GEN_BINARY_FILE]);
 
     Ok(())
 }
