@@ -6,20 +6,12 @@
 
 use std::time::Duration;
 
-pub enum CPStatus {
-    AC,
-    WA,
-    TLE,
-    CE,
-    RTE
-}
 pub trait Language {
     fn build(&self) -> bool;
     fn execute(&self, timeout: u32) -> StatusResponse;
     fn set_stdio(&mut self, stdin: &str);
 }
 
-
 pub struct StatusResponse {
     pub time: Duration,
     pub status: CPStatus
@@ -32,4 +24,33 @@ impl StatusResponse {
             status
         }
     }
-}
\ No newline at end of file
+}
+
+#[derive(PartialEq)]
+pub enum CPStatus {
+    AC,
+    WA,
+    TLE,
+    CE,
+    RTE
+}
+
+pub fn is_accepted(status: &CPStatus) -> bool {
+    *status == CPStatus::AC
+}
+
+pub fn is_wrong_answer(status: &CPStatus) -> bool {
+    *status == CPStatus::WA
+}
+
+pub fn is_time_limit_exceeded(status: &CPStatus) -> bool {
+    *status == CPStatus::TLE
+}
+
+pub fn is_compiled_error(status: &CPStatus) -> bool {
+    *status == CPStatus::CE
+}
+
+pub fn is_runtime_error(status: &CPStatus) -> bool {
+    *status == CPStatus::RTE
+}
